@@ -23,6 +23,8 @@ import socket
 import threading
 import unittest
 import urlparse
+import urllib
+from restclient.utils import to_bytestring
 
 HOST = socket.getfqdn('127.0.0.1')
 PORT = (os.getpid() % 31000) + 1024
@@ -34,16 +36,20 @@ class HTTPTestHandler(BaseHTTPRequestHandler):
         BaseHTTPRequestHandler.__init__(self, request, client_address, server)
         
     def do_GET(self):
-        self.parsed_uri = urlparse.urlparse(self.path)
+        self.parsed_uri = urlparse.urlparse(urllib.unquote(self.path))
         self.query = {}
         for k, v in cgi.parse_qsl(self.parsed_uri[4]):
             self.query[k] = v.decode('utf-8')
         path = self.parsed_uri[2]
-        
 
         if path == "/":
             extra_headers = [('Content-type', 'text/plain')]
             self._respond(200, extra_headers, "welcome")
+
+        elif path == "/unicode":
+            extra_headers = [('Content-type', 'text/plain')]
+            self._respond(200, extra_headers, u"éàù@")
+
         elif path == "/json":
             content_type = self.headers.get('content-type', 'text/plain')
             if content_type != "application/json":
@@ -51,7 +57,15 @@ class HTTPTestHandler(BaseHTTPRequestHandler):
             else:
                 extra_headers = [('Content-type', 'text/plain')]
                 self._respond(200, extra_headers, "ok")
-                
+
+        elif path == "/éàù":
+            extra_headers = [('Content-type', 'text/plain')]
+            self._respond(200, extra_headers, "ok")
+
+        elif path == "/test":
+            extra_headers = [('Content-type', 'text/plain')]
+            self._respond(200, extra_headers, "ok")
+
         elif path == "/query":
             test = self.query.get("test", False)
             if test and test == "testing":
@@ -98,6 +112,21 @@ class HTTPTestHandler(BaseHTTPRequestHandler):
             content_length = int(self.headers.get('Content-length', '-1'))
             body = self.rfile.read(content_length)
             self._respond(200, extra_headers, body)
+
+        elif path == "/bytestring":
+            content_type = self.headers.get('content-type', 'text/plain')
+            extra_headers.append(('Content-type', content_type))
+            content_length = int(self.headers.get('Content-length', '-1'))
+            body = self.rfile.read(content_length)
+            self._respond(200, extra_headers, body)
+
+        elif path == "/unicode":
+            content_type = self.headers.get('content-type', 'text/plain')
+            extra_headers.append(('Content-type', content_type))
+            content_length = int(self.headers.get('Content-length', '-1'))
+            body = self.rfile.read(content_length)
+            self._respond(200, extra_headers, body)
+
         elif path == "/json":
             content_type = self.headers.get('content-type', 'text/plain')
             if content_type != "application/json":
@@ -162,7 +191,7 @@ class HTTPTestHandler(BaseHTTPRequestHandler):
         for k, v in extra_headers:
             self.send_header(k, v)
         self.end_headers()
-        self.wfile.write(body)
+        self.wfile.write(to_bytestring(body))
         self.wfile.close()
 
     def finish(self):
