@@ -25,8 +25,9 @@ import urlparse
 import urllib2
 
 from restclient.transport import HTTPLib2Transport
-from restclient.rest import Resource, RestClient, RequestFailed, \
-ResourceNotFound, Unauthorized, RequestError
+from restclient.rest import Resource, RestClient
+from restclient.errors import RequestFailed, ResourceNotFound, \
+Unauthorized, RequestError
 
 
 from _server_test import HOST, PORT, run_server_test
